using System;
using UnityEngine;
using UnityEngine.Events;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace Mod
{
    public class Mod
    {
        static List<PersonBehaviour> NoPain = new List<PersonBehaviour>();
        static List<PersonBehaviour> Invincibling = new List<PersonBehaviour>();
        static List<PersonBehaviour> NoDrown = new List<PersonBehaviour>();
        static List<PersonBehaviour> Suffocating = new List<PersonBehaviour>();
        static List<PersonBehaviour> NoBurning = new List<PersonBehaviour>();

        static SyringeBehaviour deathS = new DeathSyringe();
        static SyringeBehaviour lifeS = new LifeSyringe();
        static SyringeBehaviour knockoutS = new KnockoutSyringe();  
        static SyringeBehaviour zombieS = new ZombieSyringe();
        
        static Color originalColor = Liquid.GetLiquid(lifeS.GetLiquidID()).Color;
        static Vector2 limits = knockoutS.Limits;

        public static void Main()
        {            
            ModAPI.Register<RetainEffects>();
            
            ModAPI.OnItemSpawned += (sender, args) =>
            {

                var PB = args.Instance.gameObject.GetComponent<PersonBehaviour>();
                if (!PB) {
                    return;
                }

                bool collapsed = false;
                bool exploded = false;

                foreach (var p in PB.Limbs) {
                    List<ContextMenuButton> buttonsList = p.PhysicalBehaviour.ContextMenuOptions.Buttons;

                    var knockOut = new ContextMenuButton(() => PB.Consciousness == 1f, "knockOut", "Knock out", "Knock out the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            PB.Consciousness = 0f;
                            foreach (var p2 in PB.Limbs)
                            {   
                                p2.Numbness = 1f;
                            }
                        })
                    });
                    knockOut.LabelWhenMultipleAreSelected = "Knock out";

                    var wakeUp = new ContextMenuButton(() => PB.Consciousness != 1f && PB.IsAlive(), "wakeUp", "Wake up", "Wake up the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            float i = 0f;
                            
                            foreach (var p2 in PB.Limbs) {
                                i += 1f;
                                PB.OxygenLevel += i * 10;
                                PB.Consciousness = i * 10;
                                PB.ShockLevel -= i * 10;
                                PB.PainLevel -= i * 10;
                                PB.AdrenalineLevel -= i * 10;

                                p2.Numbness = 0f;
                                p2.CirculationBehaviour.InternalBleedingIntensity = 0f;
                            }
                        })
                    });
                    wakeUp.LabelWhenMultipleAreSelected = "Wake up";

                    var collapse = new ContextMenuButton(() => collapsed == false, "collapse", "Collapse", "Collapse the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            collapsed = true;
                            foreach (var p2 in PB.Limbs)
                            {
                                p2.Health = 0f;
                                p2.CirculationBehaviour.IsPump = false;
                                p2.CirculationBehaviour.ForceSetAllLiquid(0f);
                                p2.BreakingThreshold = 0f;
                            }
                        })
                    });
                    collapse.LabelWhenMultipleAreSelected = "Collapse";

                    var kill = new ContextMenuButton(() => PB.IsAlive(), "kill", "Kill", "Kill the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            foreach (var p2 in PB.Limbs)
                            {   
                                p2.CirculationBehaviour.IsPump = false;
                                p2.CirculationBehaviour.ForceSetAllLiquid(0f);
                            }
                        })
                    });
                    kill.LabelWhenMultipleAreSelected = "Kill";

                    var revive = new ContextMenuButton(() => !Invincibling.Contains(PB), "revive", "Revive", "Revive the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {                           
                            foreach (var p2 in PB.Limbs)
                            {
                                if (p2.IsAndroid) {
                                    CirculationBehaviour circulationBehaviour = p2.CirculationBehaviour;
                                    p2.Health = p2.InitialHealth;
                                    p2.HealBone();
                                    circulationBehaviour.HealBleeding();
                                    circulationBehaviour.ClearLiquid();
                                    circulationBehaviour.AddLiquid(p2.GetOriginalBloodType(), 1f);
                                    circulationBehaviour.BloodFlow = 1f;
                                    circulationBehaviour.IsPump = circulationBehaviour.WasInitiallyPumping;
                                    p2.Person.Consciousness = 1f;
                                    p2.PhysicalBehaviour.BurnProgress *= 0f;
                                    
                                    continue;
                                }

                                p2.CirculationBehaviour.AddLiquid(Liquid.GetLiquid(lifeS.GetLiquidID()), limits.y);
                                Liquid.GetLiquid(lifeS.GetLiquidID()).OnEnterLimb(p2);
                                p2.CirculationBehaviour.RemoveLiquid(Liquid.GetLiquid(lifeS.GetLiquidID()), limits.y);
                            }                            
                        })
                    });
                    revive.LabelWhenMultipleAreSelected = "Revive";

                    var reviveInt = new ContextMenuButton(() => !Invincibling.Contains(PB), "reviveInternal", "Revive (internal only)", "Revive the selected human internally", new UnityAction[1]
                    {
                        (UnityAction) (() => {                           
                            foreach (var p2 in PB.Limbs)
                            {
                                var limb = p2;
                                limb.HealBone();
                                limb.Health = limb.InitialHealth;
                                limb.Numbness = 0f;
                                limb.CirculationBehaviour.HealBleeding();
                                limb.CirculationBehaviour.IsPump = limb.CirculationBehaviour.WasInitiallyPumping;
                                limb.CirculationBehaviour.BloodFlow = 1f;
                                limb.CirculationBehaviour.AddLiquid(limb.GetOriginalBloodType(), Mathf.Max(0f, 1f - limb.CirculationBehaviour.GetAmount(limb.GetOriginalBloodType())));
                                limb.BruiseCount = 0;
                                limb.CirculationBehaviour.GunshotWoundCount = 0;
                                limb.CirculationBehaviour.StabWoundCount = 0;
                                if (limb.RoughClassification == LimbBehaviour.BodyPart.Head)
                                {
                                    limb.Person.Consciousness = 1f;
                                    limb.Person.ShockLevel = 0f;
                                    limb.Person.PainLevel = 0f;
                                    limb.Person.OxygenLevel = 1f;
                                    limb.Person.AdrenalineLevel = 1f;
                                }
                            }                            
                        })
                    });
                    reviveInt.LabelWhenMultipleAreSelected = "Revive (internal only)";

                    var disablePain = new ContextMenuButton(() => PB.IsAlive() && !NoPain.Contains(PB), "disablePain", "Disable pain", "Disable the pain of the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            PB.PainLevel = 0.0f;
                            PB.ShockLevel = 0.0f;
                            NoPain.Add(PB);
                        })
                    });
                    disablePain.LabelWhenMultipleAreSelected = "Disable pain";

                    var enablePain = new ContextMenuButton(() => PB.IsAlive() && NoPain.Contains(PB), "enablePain", "Enable pain", "Enable the pain of the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => NoPain.Remove(PB))
                    });
                    enablePain.LabelWhenMultipleAreSelected = "Enable pain";

                    var disableDrowning = new ContextMenuButton(() => PB.IsAlive() && !NoDrown.Contains(PB) && !Suffocating.Contains(PB), "disableDrowning", "Disable drowning", "Disable drowning of the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => NoDrown.Add(PB))
                    });
                    disableDrowning.LabelWhenMultipleAreSelected = "Disable drowning";

                    var enableDrowning = new ContextMenuButton(() => PB.IsAlive() && NoDrown.Contains(PB) && !Suffocating.Contains(PB), "enableDrowning", "Enable drowning", "Enable drowning of the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => NoDrown.Remove(PB))
                    });
                    enableDrowning.LabelWhenMultipleAreSelected = "Enable drowning";

                    var zombify = new ContextMenuButton(() => !p.IsZombie, "zombify", "Zombify", "Zombify the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            foreach (var p2 in PB.Limbs) {
                                p2.CirculationBehaviour.AddLiquid(Liquid.GetLiquid(zombieS.GetLiquidID()), limits.y);
                                Liquid.GetLiquid(zombieS.GetLiquidID()).OnEnterLimb(p2);
                                p2.CirculationBehaviour.RemoveLiquid(Liquid.GetLiquid(zombieS.GetLiquidID()), limits.y);
                            }
                        })
                    });
                    zombify.LabelWhenMultipleAreSelected = "Zombify";

                    var disableBurning = new ContextMenuButton(() => !NoBurning.Contains(PB), "disableBurning", "Disable burning", "Disable burning of the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            NoBurning.Add(PB);
                            foreach (var p2 in PB.Limbs) {
                                var phys = p2.GetComponent<PhysicalBehaviour>();
                                var properties = phys.Properties.ShallowClone();
                                properties.Flammability = 0f;
                                phys.Properties = properties;
                            }
                        })
                    });
                    disableBurning.LabelWhenMultipleAreSelected = "Disable burning";

                    var enableBurning = new ContextMenuButton(() => NoBurning.Contains(PB), "enableBurning", "Enable burning", "Enable burning of the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            NoBurning.Remove(PB);
                            foreach (var p2 in PB.Limbs) {
                                var phys = p2.GetComponent<PhysicalBehaviour>();
                                var properties = phys.Properties.ShallowClone();
                                properties.Flammability = 1f;
                                phys.Properties = properties;
                            }
                        })
                    });
                    enableBurning.LabelWhenMultipleAreSelected = "Enable burning";
                    
                    var suffocate = new ContextMenuButton(() => PB.IsAlive() && !NoDrown.Contains(PB) && !Suffocating.Contains(PB), "suffocate", "Suffocate", "Suffocate the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            PB.OxygenLevel = 0f;
                            PB.PainLevel = 1f;
                            Suffocating.Add(PB);
                        })
                    });
                    suffocate.LabelWhenMultipleAreSelected = "Suffocate";

                    var unsuffocate = new ContextMenuButton(() => PB.IsAlive() && !NoDrown.Contains(PB) && Suffocating.Contains(PB), "unSuffocate", "Unsuffocate", "Unsuffocate the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            PB.OxygenLevel = 1f;
                            Suffocating.Remove(PB);
                        })
                    });
                    unsuffocate.LabelWhenMultipleAreSelected = "Unsuffocate";

                    var explode = new ContextMenuButton(() => exploded == false, "explode", "Explode", "Explode the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            exploded = true;
                            foreach (var p2 in PB.Limbs) {
                                p2.CirculationBehaviour.AddLiquid(p2.GetOriginalBloodType(), 15f);
                            }
                        })
                    });
                    explode.LabelWhenMultipleAreSelected = "Explode";

                    var invincible = new ContextMenuButton(() => !Invincibling.Contains(PB), "invincible", "Make invincible", "Make the selected human invincible", new UnityAction[1]
                    {
                        (UnityAction) (() => Invincibling.Add(PB))
                    });
                    invincible.LabelWhenMultipleAreSelected = "Make invincible";

                    var uninvincible = new ContextMenuButton(() => Invincibling.Contains(PB), "unInvincible", "Remove invincibility", "Remove the invincibility of the selected human", new UnityAction[1]
                    {
                        (UnityAction) (() => {
                            Invincibling.Remove(PB);

                            foreach (var p2 in PB.Limbs) {
                                p2.CirculationBehaviour.ClearLiquid();
                                p2.CirculationBehaviour.AddLiquid(p2.GetOriginalBloodType(), 1f);
                            }
                        })
                    });
                    uninvincible.LabelWhenMultipleAreSelected = "Remove invincibility";

                    buttonsList.Add(knockOut);
                    buttonsList.Add(wakeUp);
                    buttonsList.Add(collapse);
                    buttonsList.Add(kill);
                    buttonsList.Add(revive);
                    buttonsList.Add(reviveInt);
                    //buttonsList.Add(explode); //can't be fixed?
                    buttonsList.Add(zombify);
                    buttonsList.Add(invincible);
                    buttonsList.Add(uninvincible);
                    buttonsList.Add(disablePain);
                    buttonsList.Add(enablePain);
                    buttonsList.Add(disableDrowning);
                    buttonsList.Add(enableDrowning);
                    buttonsList.Add(disableBurning);
                    buttonsList.Add(enableBurning);
                    buttonsList.Add(suffocate);
                    buttonsList.Add(unsuffocate);
                }
            };

            ModAPI.OnItemRemoved += (sender, args) =>
            {
                var PB = args.Instance.gameObject.GetComponent<PersonBehaviour>();
                if (!PB) {
                    return;
                }

                if (NoPain.Contains(PB)) {
                    NoPain.Remove(PB);
                }

                if (NoDrown.Contains(PB)) {
                    NoDrown.Remove(PB);
                }

                if (Suffocating.Contains(PB)) {
                    Suffocating.Remove(PB);
                }
            };
        }

        public class RetainEffects : MonoBehaviour
        {
            public void Update()
            {
                foreach (PersonBehaviour PB in NoPain) {
                    if (!PB) {
                        NoPain.Remove(PB);
                    }

                    PB.PainLevel = 0.0f;
                    PB.ShockLevel = 0.0f;
                }

                foreach (PersonBehaviour PB in NoDrown) {
                    if (!PB) {
                        NoDrown.Remove(PB);
                    }

                    PB.OxygenLevel = 1f;
                }

                foreach (PersonBehaviour PB in Suffocating) {
                    if (!PB) {
                        Suffocating.Remove(PB);
                    }

                    PB.OxygenLevel = 0f;
                    PB.PainLevel = 1f;
                }

                foreach (PersonBehaviour PB in NoBurning) {
                    if (!PB) {
                        NoBurning.Remove(PB);
                    }

                    foreach (var p2 in PB.Limbs) {
                        p2.InternalTemperature = p2.BodyTemperature;
                        p2.GetComponent<PhysicalBehaviour>().BurnProgress = 0f;
                        p2.GetComponent<SkinMaterialHandler>().RottenProgress = 0f;
                    }
                }

                foreach (PersonBehaviour PB in Invincibling) {
                    if (!PB) {
                        Invincibling.Remove(PB);
                    }

                    foreach (var p2 in PB.Limbs)
                    {
                        if (p2.Health >= p2.InitialHealth - 1.0f) {
                            continue;
                        }

                        if (p2.IsAndroid) {
                            p2.Health = p2.InitialHealth;
                            p2.HealBone();
                            p2.CirculationBehaviour.HealBleeding();
                            p2.CirculationBehaviour.ClearLiquid();
                            p2.CirculationBehaviour.AddLiquid(p2.GetOriginalBloodType(), 1f);
                            p2.CirculationBehaviour.BloodFlow = 1f;
                            p2.CirculationBehaviour.IsPump = p2.CirculationBehaviour.WasInitiallyPumping;
                            p2.Person.Consciousness = 1f;
                            p2.PhysicalBehaviour.BurnProgress *= 0f;
                        } else {
                            //p2.CirculationBehaviour.ClearLiquid();
                            p2.CirculationBehaviour.AddLiquid(Liquid.GetLiquid(lifeS.GetLiquidID()), limits.y);
                            Liquid.GetLiquid(lifeS.GetLiquidID()).OnEnterLimb(p2);
                            p2.CirculationBehaviour.RemoveLiquid(Liquid.GetLiquid(lifeS.GetLiquidID()), limits.y);
                        }
                    }
                }
            }
        }
    }
}